
package pkg05_jobshop_inst_litterature_v2;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.util.ESat;
import java.io.*;
import java.util.Date;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.Solver;

public class Main {
 public static void main(String[] args) {
  // les données du problème
  // -----------------------
  
  int n=-1;   // 3 jobs
  int m=-1;   // 3 machines
  int h = 670; // horizon de taille 100        
  int[][] M = null;
  int[][] P = null;   
  int[][] Tab = null;   
        
  // lecture du fichier
  try{
      File  source;
      source = new File(".\\instance\\la01.txt");

      FileReader Input=new FileReader(source);
      StreamTokenizer   Lecteur;
      Lecteur=new StreamTokenizer (Input);
      Lecteur.parseNumbers();

      int nombre_ligne=-1;

      // nb ligne        
      Lecteur.nextToken();
      if (Lecteur.ttype==Lecteur.TT_NUMBER)
      {    
        nombre_ligne=(int) Lecteur.nval;
        //  System.out.println("Nombre lu = "+nombre_ligne);
        n=nombre_ligne;
      }

      // nb colonne
      int nombre_colonne=-1;
      Lecteur.nextToken();
      if (Lecteur.ttype==Lecteur.TT_NUMBER)
      {    
        nombre_colonne=(int) Lecteur.nval;
        //   System.out.println("Nombre lu = "+nombre_colonne);
        m=nombre_colonne;
      }
            
      // creation
      M = new int[n][m];
      P = new int[n][m];   
      Tab = new int[n][m];   
      int compteur = 0;
            
        for (int i=0; i<nombre_ligne; i++)
        {
            for (int j=0; j<nombre_colonne; j++)
            {
                Tab[i][j] = compteur;
                compteur++;       
                // lecture machine
                Lecteur.nextToken();
                if (Lecteur.ttype==Lecteur.TT_NUMBER)
                {    
                    M[i][j]=1+(int) Lecteur.nval;
                    // System.out.println("Nombre lu = "+M[i][j]);
                }
                // lecture processing
                Lecteur.nextToken();
                if (Lecteur.ttype==Lecteur.TT_NUMBER)
                {    
                    P[i][j]=(int) Lecteur.nval;
                    //   System.out.println("Nombre lu = "+P[i][j]);
                }
            }
        }
  }catch (Exception e) {
      System.out.println(e.getMessage());
  };    
  // fin lecture
  
  Model mon_modele = new Model("JobShop");       
   
  // 2. variables
  
  IntVar Cmax = mon_modele.intVar("Cmax", 0, h);
  
  IntVar[] date_debut = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int position = Tab[i][j];
     date_debut[position] = mon_modele.intVar("St_"+i+"_"+j, 0, h);
     }
  }
  
  IntVar[] date_fin = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int position = Tab[i][j];
     date_fin[position] = mon_modele.intVar("Ft_"+i+"_"+j, 0, h);
     }
  } 


  IntVar[] durees = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
         int position = Tab[i][j];
         durees[position] = mon_modele.intVar("p_"+i+"_"+j, P[i][j]);
     }
  }

    
  BoolVar[][] b = new BoolVar [n*m][n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
        for (int u=0; u<n; u++){
            for (int v=0; v<m; v++){
                 int position1 = Tab[i][j];
                 int position2 = Tab[u][v];
                b[position1][position2] = mon_modele.boolVar("b_"+i+"_"+j+"_"+u+"_"+v);
            }
        }
     }
  }  
  
   //  date_debut2 = mon_modele.intVarMatrix(10, 10, 0,h);


  
  
  
  // contrainte 4. relation entre date de debut et de fin
  int[] coeffs = new int[3];
  coeffs[0] = 1;	//Ft
  coeffs[1] = -1;	//St
  coeffs[2] = -1;	//P

  for (int i=0; i<n; i++)
  {
    for (int j=0; j<m; j++)
     {
       int cour = Tab[i][j];
       IntVar[] colonne = new IntVar[3]; 
       colonne[0] = date_fin[cour];
       colonne[1] = date_debut[cour];
       colonne[2] = durees[cour];
       mon_modele.scalar(colonne, coeffs,"=",0).post();
     }
   }

  
  // contrainte 5 : gamme
  for (int i=0; i<n; i++){
     for (int j=1; j<m; j++){
        int prec =Tab[i][j]-1;
        int cour = Tab[i][j];
        mon_modele.arithm(date_fin[prec], "<=",date_debut[cour]).post();
     }
  }  
    
  // contrainte 6 : disjonction 
  for (int i=0; i<n-1; i++)
  {  
     for (int j=0; j<m; j++)
     {
        int ope_i = Tab[i][j];
        for (int u=i+1; u<n; u++)
        {
            for (int v=0; v<m; v++)
            {
                int ope_u = Tab[u][v];    
                if (M[i][j]==M[u][v])
                {
                    mon_modele.ifThenElse(
                        b[ope_i][ope_u]  ,
                        mon_modele.arithm(date_fin[ope_i],"<=",date_debut[ope_u]),
                        mon_modele.arithm(date_fin[ope_u],"<=",date_debut[ope_i]));
                    
                    /*
                     mon_modele.reification(
                        b[ope_i][ope_u]  ,
                        mon_modele.arithm(date_fin[ope_i],"<=",date_debut[ope_u]));
                     
                     mon_modele.reification(
                        b[ope_u][ope_i]  ,
                         mon_modele.arithm(date_fin[ope_u],"<=",date_debut[ope_i]));
                     
                     mon_modele.arithm( b[ope_i][ope_u]  , "+",b[ope_u][ope_i], "=",1).post();
                    */
                }
            }
        } 
     }
  } 
  
  // fonction objectif
  for (int i = 0; i < n; i++) 
  {
    int valeur = i*m+m-1;
    System.out.println("valeur : "+ valeur);
    mon_modele.arithm(date_fin[i*m+m-1], "<=",Cmax).post();
  }
  

  mon_modele.setObjective(Model.MINIMIZE, Cmax);    
  
  Solver mon_solveur= mon_modele.getSolver();
  Solution solution = new Solution(mon_modele);   
 
  
  
  while (mon_solveur.solve())
  {
     solution.record(); 
    // System.out.println(Cmax.toString());
     Date courant = new Date();
     
  }
  

  
if (mon_solveur.isFeasible() == ESat.TRUE){
	int cout =(int) mon_solveur.getBestSolutionValue();
	System.out.println("Cmax = " + cout);

	for (int i=0; i<n; i++){
		for(int j=0; j<m; j++){
			int position = Tab[i][j];
			int deb = solution.getIntVal(date_debut[position]);
			int fin = solution.getIntVal(date_fin[position]);
			System.out.print(deb + "/" + fin);
			System.out.print("    ");
		}
		System.out.println();
	}
}
else
	System.out.println("pas de sol.faisable");
  
  
  
  
  
  
  
 }
}
